@@ -52,7 +52,7 @@ params = {
     "search": str,
     "pageType": (str, list),
     "xpageType": (str, list),
-    "sentiment": str,
+    "sentiment": (str, list),
     "siteGroup": list,  # user passes in a string which gets converted to a list of ids
     "xsiteGroup": list,  # user passes in a string which gets converted to a list of ids
     "domain": str,
@@ -92,10 +92,10 @@ params = {
     "latitudeMax": int,
     "longitudeMin": int,
     "longitudeMax": int,
-    "status": str,
-    "xstatus": str,
-    "priority": str,
-    "xpriority": str,
+    "status": (str, list),
+    "xstatus": (str, list),
+    "priority": (str, list),
+    "xpriority": (str, list),
     "checked": bool,
     "assigned": str,
     "xassigned": str,
@@ -259,13 +259,13 @@ mutable = {
     "removeTag": list,
     "addCategories": list,
     "removeCategories": list,
-    "priority": str,
-    "removePriority": str,
-    "status": str,
-    "removeStatus": str,
+    "priority": list,
+    "removePriority": list,
+    "status": list,
+    "removeStatus": list,
     "assignment": str,
     "removeAssignment": str,
-    "sentiment": str,
+    "sentiment": list,
     "checked": bool,
     "starred": bool,
     "location": str,
